/* Copy Utility
 *
 * Version 1.0
 * 12/19/2020
 *
 * cop.c
 */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>

#define PROG_HELP \
"Copy Utility: Copy files\n\
\n\
cop [-H] [-v] <file ...> <dest>\n\
\n\
\t[-H] - Display program syntax and options\n\
\t[-v] - Verbose mode; output details of operation\n"

#define OPT_HELP	'H'
#define OPT_VERBOSE	'v'

char options[] = "";

enum
{
	_NONE,
	_FILE,
	_DIR
};

int inopts(char opt)
{
	static int sr;
	sr = 0;
	int z;

	for (z = 0; z <= strlen(options); z++)
	{
		if (options[z] == opt)
		{
			sr = 1;
			break;
		} else {
			continue;
		}
	}

	return sr;
}

int ftype(char file[])
{
	struct stat fstat;
	int s = stat(file, &fstat);
	if (s != 0)
	{
		return _NONE;
	}
	if (S_ISREG (fstat.st_mode)) 
	{
		return _FILE;
	} else if (S_ISDIR (fstat.st_mode)) 
	{
		return _DIR;
	} else {
		return _NONE;
	}
}

int copyfile(char* afile, char* bfile)
{
	char a;
	if (ftype(afile) == _NONE)
	{
		return 1;
	} else if (ftype(bfile) == _FILE)
	{
		return 2;
	}

	FILE *aptr, *bptr;

	aptr = fopen(afile, "r");
	bptr = fopen(bfile, "w");

	if (!bptr)
	{
		fclose(aptr);
		return 0;
	}

	rewind(aptr);

	while (1)
	{
                a = fgetc(aptr);
		if (feof(aptr)) {
			break;
		}
		fputc(a, bptr);
	}
	return 1;

	fclose(aptr);
	fclose(bptr);
}

int main(int argc, char *argv[])
{
	int i;
	int p = 1;
	int r;
	int is_dir;

	if (argc >= 2 && argv[1][0] == '-')
	{
		p = 2;

		for (r = 1; r < strlen(argv[1]); r++)
		{
			options[r - 1] = argv[1][r];
		}
	}

	if (inopts(OPT_HELP))
	{
		printf(PROG_HELP);
		exit(0);
	}

	if (argc < 3 || (argc <= 3 && argv[1][0] == '-'))
	{
		printf("Usage: %s [-H] [-v] <file ..> <dest>\n", argv[0]);
		exit(1);
	}

	for (i = p; i <= (argc - 2); i++)
	{
		if (!ftype(argv[i]))
		{
			printf("%s: %s: No such file or directory\n", argv[0], argv[1]);
			continue;
		}

		if (ftype(argv[(argc - 1)]) == _FILE)
		{
			printf("%s: Destination \"%s\" already exists\n", argv[0], argv[(argc - 1)]);
			exit(1);
		}

		if (ftype(argv[(argc - 1)]) == _DIR)
		{
			is_dir = 1;
		}

		if (argc > 4 && ftype(argv[(argc - 1)]) != _DIR)
		{
			printf("%s: %s is not a directory\n", argv[0], argv[(argc - 1)]);
			exit(1);
		}

		if (!strcmp(argv[i], argv[(argc - 1)]))
		{
			printf("%s: Cannot copy %s to itself\n", argv[0], argv[i]);
			continue;
		}

		char dest[(strlen(argv[(argc - 2)]) + strlen(argv[i]))];

		if (is_dir == 1)
		{
			sprintf(dest, "%s/%s", argv[(argc - 1)], argv[i]);
		} else {
			sprintf(dest, "%s", argv[(argc - 1)]);
		}

		if (copyfile(argv[i], dest) == 1)
		{
			if (inopts(OPT_VERBOSE))
			{
				printf("%s -> %s\n", argv[i], dest);
			}
		} else {
			printf("%s: error copying %s to \"%s\": %s\n", argv[0], argv[i], dest, strerror(errno));
		}
	}

	exit(0);
}
