/* Insert Line Utility
 *
 * Version 1.0
 * 12/19/2020
 *
 * il.c
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>

#define PROG_HELP \
"Insert Line Utility: Insert a line into a file\n\
\n\
il [-H] <file> <before-line> '<string>'\n\
\n\
\t[-H] - Display program syntax and options\n\
\n\
\t<string> should be enclosed in single quotes or double quotes.\n\
\n\
\tIf <before-line> is out of range, the operation will fail.\n"

#define _USAGE "Usage: %s [-H] <file> <before-line> <string>\n"

#define OPT_HELP	'H'

char options[] = "";

enum
{
	_NONE,
	_FILE,
	_DIR
};

int inopts(char opt)
{
	static int sr;
	sr = 0;
	int z;

	for (z = 0; z <= strlen(options); z++)
	{
		if (options[z] == opt)
		{
			sr = 1;
			break;
		} else {
			continue;
		}
	}

	return sr;
}

int ftype(char file[])
{
	struct stat fstat;
	int s = stat(file, &fstat);
	if (s != 0)
	{
		return _NONE;
	}
	if (S_ISREG (fstat.st_mode)) 
	{
		return _FILE;
	} else if (S_ISDIR (fstat.st_mode)) 
	{
		return _DIR;
	} else {
		return _NONE;
	}
}

int copyfile(char* afile, char* bfile)
{
	char a;
	if (ftype(afile) == _NONE)
	{
		return 1;
	} else if (ftype(bfile) == _FILE)
	{
		return 2;
	}

	FILE *aptr, *bptr;

	aptr = fopen(afile, "r");
	bptr = fopen(bfile, "w");

	rewind(aptr);

	while (1)
	{
                a = fgetc(aptr);
		if (feof(aptr)) {
			break;
		}
		fputc(a, bptr);
	}
	return 1;

	fclose(aptr);
	fclose(bptr);
}

int insline(char tfile[], int ln, char* string)
{
	FILE *fptr;
	FILE *xptr;
	char tmpfile[128];
	char buffer[512];
	int line = 0;

	sprintf(tmpfile, "/tmp/._%s.tmp", tfile);

	fptr = fopen(tfile, "r");
	xptr = fopen(tmpfile, "w");

	if (!fptr)
	{
		fclose(xptr);
		return 1;
	}

	rewind(fptr);

	while (1)
	{
		line++;
		fgets(buffer, 512, fptr);

		if (feof(fptr))
		{
			break;
		}

		if (line != ln)
		{
			fprintf(xptr, "%s", buffer);
		} else if (line == ln) {
			fprintf(xptr, "%s\n", string);
			fprintf(xptr, "%s", buffer);
		}
	}

	fclose(fptr);
	fclose(xptr);

	remove(tfile);
	copyfile(tmpfile, tfile);
	remove(tmpfile);

	return 0;
}

int main(int argc, char *argv[])
{
	int s;
	int p = 1;
	int r;

	if (argc >= 2 && argv[1][0] == '-')
	{
		p = 2;

		for (r = 1; r < strlen(argv[1]); r++)
		{
			options[r - 1] = argv[1][r];
		}
	}

	if (inopts(OPT_HELP))
	{
		printf(PROG_HELP);
		exit(0);
	}

	if (argc != 4)
	{
		printf(_USAGE, argv[0]);
		exit(1);
	}

	if (ftype(argv[p]) == _DIR)
	{
		printf("%s: %s: Is a directory\n", argv[0], argv[p]);
		exit(1);
	} else if (ftype(argv[p]) == _NONE) {
		printf("%s: %s: File not found\n", argv[0], argv[p]);
		exit(1);
	}
	
	if (atoi(argv[p + 1]) < 1)
	{
		printf("%s: Line number must be an integer above 0\n", argv[0]);
		exit(1);
	}

	s = insline(argv[p], atoi(argv[p + 1]), argv[p + 2]);

	if (s > 0)
	{
		printf("%s: Operation failed\n", argv[0]);
		exit(1);
	} else {
		exit(0);
	}
}
