/* Make Directory Utility
 *
 * Version 1.0
 * 12/19/2020
 *
 * mkd.c
 */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>

#define PROG_HELP \
"Make Directory Utility: Create directories\n\
\n\
mkd [-H] [-v] <dir ...>\n\
\n\
\t[-H] - Display program syntax and options\n\
\t[-v] - Verbose mode; output details of operation\n"

#define OPT_HELP	'H'
#define OPT_VERBOSE	'v'

char options[] = "";

enum
{
	_NONE,
	_FILE,
	_DIR
};

int inopts(char opt)
{
	static int sr;
	sr = 0;
	int z;

	for (z = 0; z <= strlen(options); z++)
	{
		if (options[z] == opt)
		{
			sr = 1;
			break;
		} else {
			continue;
		}
	}

	return sr;
}

int ftype(char file[])
{
	struct stat fstat;
	int s = stat(file, &fstat);
	if (s != 0)
	{
		return _NONE;
	}
	if (S_ISREG (fstat.st_mode)) 
	{
		return _FILE;
	} else if (S_ISDIR (fstat.st_mode)) 
	{
		return _DIR;
	} else {
		return _NONE;
	}
}

int main(int argc, char *argv[])
{
	int i;
	int p = 1;
	int r;

	if (argc >= 2 && argv[1][0] == '-')
	{
		p = 2;

		for (r = 1; r < strlen(argv[1]); r++)
		{
			options[r - 1] = argv[1][r];
		}
	}

	if (inopts(OPT_HELP))
	{
		printf(PROG_HELP);
		exit(0);
	}

	if (argc < 2 || (argc == 2 && argv[1][0] == '-'))
	{
		printf("Usage: %s [-H] [-v] <dir-name ...>\n", argv[0]);
		exit(1);
	}

	for (i = p; i <= (argc - 1); i++)
	{
		if (ftype(argv[i]) == _DIR)
		{
			printf("%s: %s: Directory already exists\n", argv[0], argv[i]);
			continue;
		}


		if (mkdir(argv[i], 0755) == 0)
		{
			if (inopts(OPT_VERBOSE))
			{
				printf("Directory %s created\n", argv[i]);
			}
		} else {
			printf("%s: Failed to create \"%s\" (%s)\n", argv[0], argv[i], strerror(errno));
		}
	}

	exit(0);
}
