/* Message Client Utility
 *
 * Version 1.0
 * 12/25/2020
 *
 * msgcli.c
 */

#include <arpa/inet.h>
#include <ctype.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <unistd.h>

#define PROG_HELP \
"Message Client Utility: Simple telnet client which reads text from a message server\n\
\n\
msgcli [-H] <host> <port>\n\
\n\
\t[-H] - Display program syntax and options\n\
\n\
\t<host> must be a valid domain name or IP address.\n\
\t<port> must be a valid TCP port number.\n"

char * get_ip(char hostname[])
{
	static char ip[16];
	struct hostent *ent;
	struct in_addr **addr;

	if ((ent = gethostbyname(hostname)) == NULL)
	{
		return "(not found)";
	}

	addr = (struct in_addr **) ent->h_addr_list;

	strcpy(ip, inet_ntoa(*addr[0]));

	return ip;
}

int main(int argc, char *argv[])
{

	if (argc > 1 && argv[1][0] == '-' && argv[1][1] == 'H')
	{
		printf(PROG_HELP);
		exit(0);
	}

	if (argc != 3)
	{
		printf("Usage: %s [-H] <host> <port>\n", argv[0]);
		exit(1);
	}

	int sock;
	int rd;
	int c;
	struct sockaddr_in server;
	char buffer[1024];

	if ((sock = socket(AF_INET, SOCK_STREAM, 0)) == 0)
	{
		printf("%s: failed to create socket\n", argv[0]);
		exit(1);
	}

	server.sin_family = AF_INET;
	server.sin_port = htons(atoi(argv[2]));

	if ((inet_pton(AF_INET, get_ip(argv[1]), &server.sin_addr)) < 0)
	{
		printf("%s: address not supported\n", argv[0]);
		exit(1);
	}

	if ((connect(sock, (struct sockaddr *)&server, sizeof(server))) < 0)
	{
		printf("%s: connection failed\n", argv[0]);
		exit(1);
	}

	while (1)
	{
		rd = read(sock, buffer, 1024);

		if (rd <= 0)
			break;

		for (c = 0; c < rd; c++)
		{
			putchar(buffer[c]);
		}
		strcpy(buffer, "");
	}

	return 0;
}
