/* Slasher Utility
 *
 * Version 1.0
 * 12/19/2020
 *
 * slasher.c
 */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <time.h>

#define PROG_HELP \
"Slasher Utility: Corrupt and optionally delete files\n\
\n\
slasher [-H] [-vdr] <file ...>\n\
\n\
\t[-H] - Display program syntax and options\n\
\t[-v] - Verbose mode; output details of operation\n\
\t[-d] - Delete the file after overwriting it\n\
\t[-r] - Overwrite with a random number of bytes (up to 10240)\n\
\n\
\tWhen '-r' is not used, the file size does not change.\n"

#define MAX_RAND_LEN	10240

#define OPT_HELP	'H'
#define OPT_VERBOSE	'v'
#define OPT_DELETE	'd'
#define OPT_RSIZE	'r'

char options[] = "";

enum
{
	_NONE,
	_FILE,
	_DIR
};

int inopts(char opt)
{
	static int sr;
	sr = 0;
	int z;

	for (z = 0; z <= strlen(options); z++)
	{
		if (options[z] == opt)
		{
			sr = 1;
			break;
		} else {
			continue;
		}
	}

	return sr;
}

int ftype(char file[])
{
	struct stat fstat;
	int s = stat(file, &fstat);
	if (s != 0)
	{
		return _NONE;
	}
	if (S_ISREG (fstat.st_mode)) 
	{
		return _FILE;
	} else if (S_ISDIR (fstat.st_mode)) 
	{
		return _DIR;
	} else {
		return _NONE;
	}
}

int fsize(char file[])
{
	struct stat fstat;
	static int sz;
	stat(file, &fstat);
	sz = fstat.st_size;
	return sz;
}

int irandom(int max) 
{
	srand(time(0));
	static int r;
	r = (rand() % (max));
	return r + 1;
}

int main(int argc, char *argv[])
{
	int a = 0;
	int i, q;
	int fs;
	int p = 1;
	int r;
	char c;
	FILE *fptr;
	FILE *rptr;

	if (argc >= 2 && argv[1][0] == '-')
	{
		p = 2;

		for (r = 1; r < strlen(argv[1]); r++)
		{
			options[r - 1] = argv[1][r];
		}
	}

	if (inopts(OPT_HELP))
	{
		printf(PROG_HELP);
		exit(0);
	}

	if (argc < 2 || (argc == 2 && argv[1][0] == '-'))
	{
		printf("Usage: %s [-H] [-vdr] <file ...>\n", argv[0]);
		exit(1);
	}

	for (i = p; i <= (argc - 1); i++)
	{
		if (ftype(argv[i]) == _NONE)
		{
			printf("%s: %s: File not found\n", argv[0], argv[i]);
			continue;
		} else if (ftype(argv[i]) == _DIR) {
			printf("%s: %s: Is a directory\n", argv[0], argv[i]);
			continue;
		} else {
			if (inopts(OPT_RSIZE))
			{
				fs = irandom(MAX_RAND_LEN);
			} else {
				fs = fsize(argv[i]);
			}

			fptr = fopen(argv[i], "w");
			rptr = fopen("/dev/urandom", "r");

			while (1)
			{
				c = fgetc(rptr);
				fputc(c, fptr);
				a++;
				if (a >= fs)
				{
					break;
				}
			}

			fclose(fptr);
			fclose(rptr);

			if (inopts(OPT_VERBOSE))
			{
				printf("Destroyed %s (%d bytes written)\n", argv[i], fs);
			}

			if (inopts(OPT_DELETE))
			{
				if (remove(argv[i]) == 0)
				{
					if (inopts(OPT_VERBOSE))
					{
						printf("%s: Deleted %s\n", argv[0], argv[i]);
					}
					continue;
				} else {
					printf("%s: Cannot delete %s: %s\n", argv[0], argv[i], strerror(errno));
				}
			}
		}
	}

	exit(0);

}
