/* Write Line Utility
 *
 * Version 1.0
 * 1/16/2021
 *
 * wl.c
 */
 
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>

#define PROG_HELP \
"Write Line Utility: Write a line of text to a new file\n\
\n\
wl [-H] <file> '<text>'\n\
\n\
\t[-H] - Display program syntax and options\n\
\n\
\tThis program will not attempt to overwrite <file> if it exists.\n"

#define _USAGE "Usage: %s [-H] <file> '<text>'\n"

#define OPT_HELP	'H'

char options[8];

enum
{
	_NONE,
	_FILE,
	_DIR
};

int inopts(char opt)
{
	static int sr;
	sr = 0;
	int z;

	for (z = 0; z <= strlen(options); z++)
	{
		if (options[z] == opt)
		{
			sr = 1;
			break;
		} else {
			continue;
		}
	}

	return sr;
}

int ftype(char file[])
{
	struct stat fstat;
	int s = stat(file, &fstat);
	if (s != 0)
	{
		return _NONE;
	}
	if (S_ISREG (fstat.st_mode)) 
	{
		return _FILE;
	} else if (S_ISDIR (fstat.st_mode)) 
	{
		return _DIR;
	} else {
		return _NONE;
	}
}

int main(int argc, char *argv[])
{
	FILE *fptr;
	int r;

	if (argc >= 2 && argv[1][0] == '-')
	{
		for (r = 1; r < strlen(argv[1]); r++)
		{
			options[r - 1] = argv[1][r];
		}
	}

	if (inopts(OPT_HELP))
	{
		printf(PROG_HELP);
		exit(0);
	}

	if (argc != 3)
	{
		printf(_USAGE, argv[0]);
		exit(1);
	}

	if (ftype(argv[1]) != _NONE)
    	{
		printf("%s: %s: file already exists\n", argv[0], argv[1]);
		exit(1);
	}

	fptr = fopen(argv[1], "w");

	if (!fptr)
	{
		printf("%s: %s: %s\n", argv[0], argv[1], strerror(errno));
		exit(1);
	}

    fprintf(fptr, "%s\n", argv[2]);
    fclose(fptr);

    exit(0);

}
